class Block:
    def __init__(self, start, name):
        self.list_of_contained_genes = [start]
        self.name = "Block_%s"%name
        
    @property
    def length(self):
        return len(self.list_of_contained_genes)

    def __repr__(self):
        return "Block has %s"%self.list_of_contained_genes

import sys

try:
    fh = open(sys.argv[1])
except:
    sys.stderr.write("Usage: python 2-getLongestBlockPerBlock.py MCScan_output.csv\n")
    sys.exit(0)

header = fh.readline()
assert "Duplication" in header

assert fh.name.endswith(".csv")

out = fh.name.rstrip(".csv") + "_merged.csv"
out = open(out, "w")

new_header = ["Gene","Block","Block Start","Block End"]
out.write("\t".join(new_header) + "\n")
# pass through the file twice - once to make all blocks, once to assign genes to blocks
block_counter = 0
current_blocks = {} # key: counter, value: currently manipulated block
all_blocks = {} # key: blockname, value: block object

genes_to_blocks = {} # key: gene name, value: block it is in

for line in fh:
    ll = line.rstrip("\n\r").split("\t")
    if ll[0] == "0":
        # no block
        # close all prior blocks if they existed
        current_blocks = {}
        continue

    partners = ll[2:]
    # every second element is empty starting from 0
    # example: ['', 'maker-chrA10_contigs_placed_v81-augustus-gene-195.163-mRNA-1', '', '|\xc2\xa0|']
    partners = partners[1::2]
    for counter, element in enumerate(partners):
        if not element:
            # end of block
            continue
        # do we have a new block
        if counter not in current_blocks:
            b = Block(element, block_counter)
            current_blocks[counter] = b
            genes_to_blocks[element] = b
            all_blocks[block_counter] = b
            block_counter += 1
        else:
            b = current_blocks[counter]
            b.list_of_contained_genes.append(element)
            genes_to_blocks[element] = b

    last_partners = partners

fh.seek(0)
header = fh.readline()

for line in fh:
    ll = line.rstrip("\n\r").split("\t")
    if ll[0] == "0":
        out.write(ll[1] + "\n")
        continue
    partners = ll[2:]
    partners = partners[1::2]
    longest_block = ""
    longest_block_length = 0
    for element in partners:
        if not element: continue
        block_this_gene_is_on = genes_to_blocks[element]
        length_of_this_block = block_this_gene_is_on.length
        if length_of_this_block > longest_block_length:
            longest_block = block_this_gene_is_on
            longest_block_length = length_of_this_block
    out.write(ll[1] + "\t" + longest_block.name + "\t" + longest_block.list_of_contained_genes[0] + "\t" + longest_block.list_of_contained_genes[-1] + "\n")
