import os
import sys
try:
    fh = open(sys.argv[1])
    blast_fh = open(sys.argv[2])
except:
    sys.stderr.write("Usage: 0-splitGff.py File_with_all_expected_ranges.csv File_with_Blast_results.txt\n")
    sys.exit(0)

out = open(".".join(fh.name.split(".")[:1]) + "_non_hitting_genes_without_ranges.txt", "w")

transl_dict = {'Tapidor_chrA04_v6.3': 'ta04', 'Tapidor_chrA02_v6.3': 'ta02', 'chrC04_contigs_placed_v81': 'da14', 'chrC02_contigs_placed_v81': 'da12', 'Tapidor_chrC07_v6.3': 'ta17', 'chrC06_contigs_placed_v81': 'da16', 'Tapidor_chrA07_v6.3': 'ta07', 'Tapidor_chrC06_v6.3': 'ta16', 'chrA03_contigs_placed_v81': 'da03', 'chrA06_contigs_placed_v81': 'da06', 'chrC05_contigs_placed_v81': 'da15', 'Tapidor_chrA09_v6.3': 'ta09', 'Tapidor_chrC05_v6.3': 'ta15', 'chrA07_contigs_placed_v81': 'da07', 'Tapidor_chrA06_v6.3': 'ta06', 'Tapidor_chrA10_v6.3': 'ta10', 'Tapidor_chrC02_v6.3': 'ta12', 'Tapidor_Unplaced_v6.3': 'ta20', 'Tapidor_chrA08_v6.3': 'ta08', 'Tapidor_chrC04_v6.3': 'ta14', 'chrA01_contigs_placed_v81': 'da01', 'Tapidor_chrC09_v6.3': 'ta19', 'Tapidor_chrC01_v6.3': 'ta11', 'Unplaced_contigs_v81': 'da20', 'chrA09_contigs_placed_v81': 'da09', 'chrC01_contigs_placed_v81': 'da11', 'Tapidor_chrC08_v6.3': 'ta18', 'chrC09_contigs_placed_v81': 'da19', 'chrA05_contigs_placed_v81': 'da05', 'chrC08_contigs_placed_v81': 'da18', 'chrA04_contigs_placed_v81': 'da04', 'chrA02_contigs_placed_v81': 'da02', 'chrA10_contigs_placed_v81': 'da10', 'Tapidor_chrA01_v6.3': 'ta01', 'chrC03_contigs_placed_v81': 'da13', 'Tapidor_chrA03_v6.3': 'ta03', 'Tapidor_chrC03_v6.3': 'ta13', 'Tapidor_chrA05_v6.3': 'ta05', 'chrC07_contigs_placed_v81': 'da17', 'chrA08_contigs_placed_v81': 'da08'}

hits_dict = {} # key: gene, value: list of ranges where it could hit
for line in blast_fh:
    ll = line.split()
    query, subject = ll[0], ll[1]
    start, end = map(int, ll[8:10])
    short_subject = transl_dict[subject]
    if query not in hits_dict:
        hits_dict[query] = [ (short_subject, start, end) ]
    else:
        hits_dict[query].append( (short_subject, start, end) )

no_hits_at_all = 0
all_hits_correct_chromosome = 0
all_hits_correct_range = 0
all_genes = 0

for line in fh:

    all_genes += 1
    ll = line.split()
    query = ll[0]
    try:
        real_hits = hits_dict[query]
    except KeyError:
        no_hits_at_all += 1
        continue
    
    expected_start = ll[4]
    if expected_start == "START":
        expected_start = 0

    expected_end = ll[-2] # could be END, in which case expected end is length of chromosome

    expected_chrom = ll[-3]
    if expected_chrom == "END":
        expected_chrom = ll[2]
    hits_on_correct_chromosome = False
    hits_in_expected_range = False

    for hit in real_hits:
        # do we have the right chromosome?
        if expected_chrom == hit[0]:

            hits_on_correct_chromosome = True
            # do we have the right range? 1 bp is enough

            actual_start = int(hit[1])
            actual_end = int(hit[2])
            expected_start = int(expected_start)
            # check whether they overlap
            # do we have an END?
            if expected_end == "END":
                # just check whether either actual_start or actual_end are bigger than expected_start
                if (actual_start > expected_start) or (actual_end > expected_start):
                    hits_in_expected_range = True
                continue

            # SANITY
            expected_start = int(expected_start)
            expected_end = int(expected_end)

            if actual_start > actual_end:
                actual_end, actual_start = actual_start, actual_end
            if expected_start > expected_end:
                expected_end, expected_start = expected_start, expected_end
            
            # now check whether both ranges overlap
            # easier to find out whether they DONT overlap
            #    hit comes before expected   OR   hit comes after expected 
            if (actual_end < expected_start) or (actual_start > expected_end):
                #out.write("NOT"
                #out.write(query, "actual", actual_start, actual_end, "expected", expected_start, expected_end
                pass
            else:
                #out.write("INSIDE"
                out.write("%s\t%s\tACTUAL RANGE FROM BLAST HIT\t%s\t%s\tEXPECTED RANGE FROM SURROUNDING BLOCKS\t%s\t%s\n"%(query, hit[0], actual_start, actual_end,expected_start, expected_end))
                hits_in_expected_range = True

    if hits_on_correct_chromosome:
        all_hits_correct_chromosome += 1
    if hits_in_expected_range:
        all_hits_correct_range += 1

out.write("# All genes seen: %s\n"%all_genes)
out.write("# All genes without any hits: %s\n"%no_hits_at_all)
out.write("# All genes with hits on correct chromosome: %s\n"%all_hits_correct_chromosome)
out.write("# All genes with hits on correct chromosome in correct range: %s\n"%all_hits_correct_range)
out.write("# All genes with hits on correct chromosome but not in correct range: %s\n"%(all_hits_correct_chromosome - all_hits_correct_range))
